import subprocess
import os
from i3situation.plugins._plugin import Plugin

__all__ = 'BatteryPlugin'


class BatteryPlugin(Plugin):
    """
    Displays information about the battery, such as its current charge and
    current status (Discharging, Full, Charging).
    """

    def __init__(self, config):
        self.options = {'format': '<charge>%',
                        'low_threshold': 20,
                        'low_color': '#FF0000',
                        'discharging_color': '#FF6103',
                        'charging_color': '#00F000',
                        'full_color': '#FFFFFF',
                        'percentage': True,
                        'battery_path': '/sys/class/power_supply/BAT0',
                        'interval': 30}

        super().__init__(config)

    def main(self):
        energy_now = int(self.get_battery_state('energy_now'))
        energy_full = int(self.get_battery_state('energy_full'))
        status = self.get_battery_state('status')

        charge = (energy_now / energy_full) * 100 if self.options['percentage'] else 1
        charge = int(charge)

        if charge < self.options['low_threshold'] and status == 'Discharging':
            self.output_options['color'] = self.options['low_color']
        elif status == 'Full':
            self.output_options['color'] = self.options['charging_color']
        elif status == 'Discharging':
            self.output_options['color'] = self.options['discharging_color']
        elif status == 'Charging':
            self.output_options['color'] = self.options['charging_color']

        output = self.options['format'].replace('<charge>', str(charge))
        output = output.replace('<status>', status)

        return self.output(output, output)

    def get_battery_state(self, prop):
        """
        Return the first line from the file located at battery_path/prop as a
        string.
        """
        with open(os.path.join(self.options['battery_path'], prop), 'r') as f:
                return f.readline().strip()



import glob
import shutil
import imp
import logging
import time
import os.path
import threading
import compileall


class MissingPlugin(Exception):
    """
    The plugin that has been mentioned in the configuration file doesn't appear
    in the application's plugin directory.
    """
    pass


class Thread(threading.Thread):
    """
    A thread that runs a plugin's main function and mutates that outputdict
    in order to return the value of the plugin's main function. This is where
    a plugin's interval is taken into account.
    """

    def __init__(self, func, interval, output_dict):
        """
        func: The function that will be executed by the thread. It is always the
        main() function of a plugin.

        interval: How often the func is run. (Time in seconds).

        output_dict: A dictionary containing dictionaries. Is in the format:
        {plugin_name: {information about plugin}, ...}
        The information about plugin contains data such as the full_text and
        formatting options.
        """
        super().__init__(group=None, daemon=True)
        self.func = func
        self.output_dict = output_dict
        self.interval = interval

    def run(self):
        """
        Calls the main function of a plugin and mutates the output dict
        with its return value. Provides an easy way to change the output
        whilst not needing to constantly poll a queue in another thread and
        allowing plugin's to manage their own intervals.
        """
        self.running = True
        while self.running:
            ret = self.func()
            self.output_dict[ret['name']] = ret
            time.sleep(self.interval)
        return

    def stop(self):
        """
        Stop the thread from running.
        The thread doesn't stop immediately, it stops once the running
        variable is checked, after the thread's interval.
        """
        self.running = False


class ThreadManager():
    """
    Keeps track of threads and enables the creation and destruction of
    threads.

    The output_dict variable is a dictionary passed into the class that is then
    mutated by the threads in order to update the output of the application.
    This means that the main thread need only display the output and not poll
    for changes in the output data. It also means that queues are not needed.
    """

    def __init__(self, output_dict):
        """
        output_dict: Same as that used in the Thread class ^^^.
        """
        self._thread_pool = []
        self.output_dict = output_dict

    def add_thread(self, func, interval):
        """
        Creates a thread, starts it and then adds it to the thread pool.

        Func: Same as in the Thread class.
        Interval: Same as in the Thread class.
        """
        t = Thread(func, interval, self.output_dict)
        t.start()
        self._thread_pool.append(t)

    def kill_all_threads(self):
        """
        Provides an easy way to graciously end all threads.
        """
        for t in self._thread_pool:
            t.stop()
        self._thread_pool = []


class PluginLoader():
    """
    Loads classes from plugin files placed in the plugin directory.
    The plugin's class name must be the only part of the __all__
    variable. Any files that should not be loaded as plugins require
    an underscore at the start of their filename, such as:
    _plugin.py
    """
    def __init__(self, dir_path, config):
        self.dir_path = dir_path
        source_path = '/'.join(os.path.dirname(__file__).split('/')[0:-1])
        source_path = os.path.join(source_path, 'plugins')
        if len(glob.glob(self.dir_path + '/*')) < len(glob.glob(source_path + '/*')) \
                or os.path.getmtime(source_path) > os.path.getmtime(self.dir_path):
            shutil.rmtree(self.dir_path)
            shutil.copytree(source_path, self.dir_path)
        self._compile_files()
        self._config = config
        self.plugins = self.refresh_files()
        self.objects = self.load_objects()

    def _compile_files(self):
        """
        Compiles python plugin files in order to be processed by the loader.
        It compiles the plugins if they have been updated or haven't yet been
        compiled.
        """
        for f in glob.glob(os.path.join(self.dir_path, '*.py')):
            # Check for compiled Python files that aren't in the __pycache__.
            if not os.path.isfile(os.path.join(self.dir_path, f + 'c')):
                compileall.compile_dir(self.dir_path, quiet=True)
                logging.debug('Compiled plugins as a new plugin has been added.')
                return
            # Recompile if there are newer plugins.
            elif os.path.getmtime(os.path.join(self.dir_path, f)) > os.path.getmtime(
                    os.path.join(self.dir_path, f + 'c')):
                compileall.compile_dir(self.dir_path, quiet=True)
                logging.debug('Compiled plugins as a plugin has been changed.')
                return

    def _load_compiled(self, file_path):
        """
        Accepts a path to a compiled plugin and returns a module object.

        file_path: A string that represents a complete file path to a compiled
        plugin.
        """
        name = os.path.splitext(os.path.split(file_path)[-1])[0]
        plugin_directory = os.sep.join(os.path.split(file_path)[0:-1])
        compiled_directory = os.path.join(plugin_directory, '__pycache__')
        # Use glob to autocomplete the filename.
        compiled_file = glob.glob(os.path.join(compiled_directory, (name + '.*')))[0]
        plugin = imp.load_compiled(name, compiled_file)
        return plugin

    def load_objects(self):
        """
        Matches the plugins that have been specified in the config file
        with the available plugins. Returns instantiated objects based upon
        the classes defined in the plugins.
        """
        objects = []
        for settings in self._config:
            if settings['plugin'] in self.plugins:
                module = self.plugins[settings['plugin']]
                # Trusts that the only item in __all__ is the name of the
                # plugin class.
                plugin_class = getattr(module, module.__all__)
                objects.append(plugin_class(settings))
                logging.debug('Loaded a plugin object based upon {0}'.format(
                    settings['plugin']))
            else:
                logging.critical('Missing plugin {0} was not found in {1}'.format(
                    settings['plugin'], self.dir_path))
                raise MissingPlugin('The plugin {0} was not found in {1}'.format(
                    settings['plugin'], self.dir_path))
        return objects

    def refresh_files(self):
        """
        Discovers the available plugins and turns each into a module object.
        This is a seperate function to allow plugins to be updated
        dynamically by other parts of the application.
        """
        plugins = {}
        _plugin_files = glob.glob(os.path.join(self.dir_path, '[!_]*.pyc'))
        for f in glob.glob(os.path.join(self.dir_path, '[!_]*.py')):
            if not any(os.path.splitext(f)[0] == os.path.splitext(x)[0]
                    for x in _plugin_files):
                logging.debug('Adding plugin {0}'.format(f))
                _plugin_files.append(f)
        for f in _plugin_files:
            plugin = self._load_compiled(f)
            plugins[plugin.__name__] = plugin
            logging.debug('Loaded module object for plugin: {0}'.format(f))
        return plugins



#!/usr/bin/python3
import logging
from i3situation.core import status


def main():
    s = status.Status()
    logging.debug('Initialised Status object')
    s.run()

if __name__ == '__main__':
    main()



import html.parser
import webbrowser
import requests
from i3situation.plugins._plugin import Plugin

__all__ = 'RedditPlugin'


class RedditPlugin(Plugin):
    """
    A plugin to display data from Reddit. The submissions on the front page can
    be displayed as well as the submissions on selected subreddits. The format
    options can contain the following keywords that will be replace at run time:

    link_flair_css_class
    approved_by
    num_reports
    num_comments
    author_flair_css_class
    link_flair_text
    ups
    author_flair_text
    permalink
    selftext
    domain
    subreddit_id
    over_18
    secure_media
    name
    media
    clicked
    score
    is_self
    stickied
    distinguished
    media_embed
    subreddit
    edited
    selftext_html
    banned_by
    likes
    created_utc
    hidden
    url
    id
    downs
    thumbnail
    author
    secure_media_embed
    title
    saved
    """

    def __init__(self, config):
        self.options = {'mode': 'front', 'color': '#FFFFFF', 'interval': 30,
                        'subreddits': ['vim', 'python'], 'username': None,
                        'password': None, 'limit': 25, 'format': '[subreddit] title ↑ups',
                        'sort': 'hot'}
        super().__init__(config)
        if isinstance(self.options['subreddits'], str):
            self.options['subreddits'] = [self.options['subreddits']]
        self.h = html.parser.HTMLParser()
        self.client = requests.session()
        self.client.headers.update({'user-agent': 'i3situation reddit plugin'})

    def main(self):
        """
        Generates an output string by replacing the keywords in the format
        string with the corresponding values from a submission dictionary.
        """
        self.manage_submissions()
        out_string = self.options['format']
        self.selected_submission = self.submissions.pop()
        for k, v in self.selected_submission.items():
            out_string = out_string.replace(k, str(v))
        return self.output(out_string, out_string)

    def login(self):
        """
        Logs into Reddit in order to display a personalised front page.
        """
        data = {'user': self.options['username'], 'passwd':
                self.options['password'], 'api_type': 'json'}
        response = self.client.post('http://www.reddit.com/api/login', data=data)
        self.client.modhash = response.json()['json']['data']['modhash']

    def manage_submissions(self):
        """
        If there are no or only one submissions left, get new submissions.
        This function manages URL creation and the specifics for front page
        or subreddit mode.
        """
        if not hasattr(self, 'submissions') or len(self.submissions) == 1:
            self.submissions = []
            if self.options['mode'] == 'front':
                # If there are no login details, the standard front
                # page will be displayed.
                if self.options['password'] and self.options['username']:
                    self.login()
                url = 'http://reddit.com/.json?sort={0}'.format(self.options['sort'])
                self.submissions = self.get_submissions(url)
            elif self.options['mode'] == 'subreddit':
                for subreddit in self.options['subreddits']:
                    url = 'http://reddit.com/r/{0}/.json?sort={1}'.format(
                        subreddit, self.options['limit'])
                    self.submissions += self.get_submissions(url)
        else:
            return

    def get_submissions(self, url):
        """
        Connects to Reddit and gets a JSON representation of submissions.
        This JSON data is then processed and returned.

        url: A url that requests for submissions should be sent to.
        """
        response = self.client.get(url, params={'limit': self.options['limit']})
        submissions = [x['data'] for x in response.json()['data']['children']]
        return submissions



import subprocess
import datetime
from i3situation.plugins._plugin import Plugin

__all__ = 'CmusPlugin'


class CmusPlugin(Plugin):

    def __init__(self, config):
        """
        Possible format options are:

        status
        file
        duration
        position
        artist
        album
        title
        date
        genre
        tracknumber
        comment
        replaygain_track_gain
        aaa_mode
        continue
        play_library
        play_sorted
        replaygain
        replaygain_limit
        replaygain_preamp
        repeat
        repeat_current
        shuffle
        softvol
        vol_left
        vol_right
        """
        self.options = {'interval': 1, 'format':
                'artist - title - position/duration'}
        super().__init__(config)

    def main(self):
        """
        A compulsary function that gets the output of the cmus-remote -Q command
        and converts it to unicode in order for it to be processed and finally
        output.
        """
        try:
            # Setting stderr to subprocess.STDOUT seems to stop the error
            # message returned by the process from being output to STDOUT.
            cmus_output = subprocess.check_output(['cmus-remote', '-Q'],
                                    stderr=subprocess.STDOUT).decode('utf-8')
        except subprocess.CalledProcessError:
            return self.output(None, None)
        if 'duration' in cmus_output:
            status = self.convert_cmus_output(cmus_output)
            out_string = self.options['format']
            for k, v in status.items():
                out_string = out_string.replace(k, v)
        else:
            out_string = None
        return self.output(out_string, out_string)

    def convert_cmus_output(self, cmus_output):
        """
        Change the newline separated string of output data into
        a dictionary which can then be used to replace the strings in the config
        format.

        cmus_output: A string with information about cmus that is newline
        seperated. Running cmus-remote -Q in a terminal will show you what
        you're dealing with.
        """
        cmus_output = cmus_output.split('\n')
        cmus_output = [x.replace('tag ', '') for x in cmus_output if not x in '']
        cmus_output = [x.replace('set ', '') for x in cmus_output]
        status = {}
        partitioned = (item.partition(' ') for item in cmus_output)
        status = {item[0]: item[2] for item in partitioned}
        status['duration'] = self.convert_time(status['duration'])
        status['position'] = self.convert_time(status['position'])
        return status

    def convert_time(self, time):
        """
        A helper function to convert seconds into hh:mm:ss for better
        readability.

        time: A string representing time in seconds.
        """
        time_string = str(datetime.timedelta(seconds=int(time)))
        if time_string.split(':')[0] == '0':
            time_string = time_string.partition(':')[2]
        return time_string



import subprocess
from i3situation.plugins._plugin import Plugin

__all__ = 'RunPlugin'


class RunPlugin(Plugin):
    """
    A plugin that runs arbitrary shell commands and outputs them to the bar.
    USE WITH CAUTION.
    """

    def __init__(self, config):
        self.options = {'command': 'echo No command',
                        'interval': 1}
        super().__init__(config)

    def main(self):
        shell_output = subprocess.check_output(self.options['command'].split(' '),
                                stderr=subprocess.STDOUT).decode('utf-8')
        return self.output(shell_output, shell_output)



import time
import subprocess
import os
from i3situation.plugins._plugin import Plugin

__all__ = 'DateTimePlugin'


class DateTimePlugin(Plugin):

    def __init__(self, config):
        self.options = {'time_zone': 'GMT', 'long_format': '%d-%m-%Y %H:%M:%S',
                'short_format': '%H:%M:%S', 'interval': 1, 'menu_command': ''}
        super().__init__(config)

    def main(self):
        cur = time.time()
        os.environ['TZ'] = self.options['time_zone']
        time.tzset()
        long_time = time.strftime(self.options['long_format'], time.localtime(cur))
        short_time = time.strftime(self.options['short_format'], time.localtime(cur))
        return self.output(long_time, short_time)



import subprocess
from i3situation.plugins._plugin import Plugin

__all__ = 'ConkyPlugin'


class ConkyPlugin(Plugin):

    def __init__(self, config):
        self.options = {'command': '$uptime', 'interval': 1,
                'config': '~/.conkyrc'}
        super().__init__(config)

    def main(self):
        try:
            out = subprocess.check_output(['conky', '-i', '1', '-t',
                self.options['command'], '-c', self.options['config']]).decode('utf8')
        except subprocess.CalledProcessError:
            out = 'A non-zero exit status was returned.'
            self.output_options['color'] = '#FF0000'
        except FileNotFoundError:
            out = 'Conky isn\'t installed.'
            self.output_options['color'] = '#FF0000'

        return self.output(out, out)



import sys
import threading
from collections import OrderedDict
import logging
import time
import json
import os
from i3situation.core import plugin_manager
from i3situation.core import config


def setup_file_logger(filename, formatting, log_level):
    """
    A helper function for creating a file logger.
    Accepts arguments, as it is used in Status and LoggingWriter.
    """
    logger = logging.getLogger()
    # If a stream handler has been attached, remove it.
    if logger.handlers:
        logger.removeHandler(logger.handlers[0])
    handler = logging.FileHandler(filename)
    logger.addHandler(handler)
    formatter = logging.Formatter(*formatting)
    handler.setFormatter(formatter)
    logger.setLevel(log_level)
    handler.setLevel(log_level)
    return logger


class Status():
    """
    Handles the running of the status utility and acts as the glue for the
    application.
    """
    def __init__(self):
        self.config = config.Config(self.discover_folder_path())
        self.output_dict = OrderedDict()
        self._config_file_path = self.config.config_file_path
        self._plugin_path = self.config.plugin_path
        self._config_mod_time = os.path.getmtime(self._config_file_path)
        self._plugin_mod_time = os.path.getmtime(self._plugin_path)
        log_formatting = ('[%(asctime)s] - %(levelname)s - %(filename)s - '
                '%(funcName)s - %(message)s', '%d/%m/%Y %I:%M:%S %p')
        self.logger = setup_file_logger(self.config.general['log_file'],
                log_formatting, self.config.general['logging_level'])

        # Redirect stderr so that it doesn't confuse i3bar by outputting to it.
        self.log_writer = self.LoggingWriter(
                self.config.general['log_file'], self.config.general['logging_level'])
        sys.stderr = self.log_writer
        logging.debug('Config loaded from {0}'.format(self._config_file_path))
        logging.debug('Plugin path is located at {0}'.format(self._plugin_path))
        logging.debug('Last config modification time is: {0}'.format(self._config_mod_time))
        logging.debug('Last plugin directory modification time is: {0}'.format(self._plugin_mod_time))
        self.output_to_bar(json.dumps({'version': 1, 'click_events': True}), False)
        self.output_to_bar('[', False)
        logging.debug('Sent initial JSON data to i3bar.')
        logging.debug('Beginning plugin loading process')
        self.loader = plugin_manager.PluginLoader(
            self._plugin_path, self.config.plugin)
        self.thread_manager = plugin_manager.ThreadManager(self.output_dict)
        # Event handling is done in another thread, so that the main thread
        # isn't stalled.
        self.event_thread = threading.Thread(target=self.handle_events)
        self.event_thread.start()

    def discover_folder_path(self):
        if os.environ.get('$XDG_CONFIG_HOME', None):
            folder_locations = [os.path.join(os.environ.get('$XDG_CONFIG_HOME'),
                'i3situation'), '/etc/i3situation']
        else:
            folder_locations = [os.path.join(os.path.expanduser('~'), '.config',
                'i3situation'), '/etc/i3situation']
        return folder_locations

    def output_to_bar(self, message, comma=True):
        """
        Outputs data to stdout, without buffering.

        message: A string containing the data to be output.
        comma: Whether or not a comma should be placed at the end of the output.
        """
        if comma:
            message += ','
        sys.stdout.write(message + '\n')
        sys.stdout.flush()

    def reload(self):
        """
        Reload the installed plugins and the configuration file. This is called
        when either the plugins or config get updated.
        """
        logging.debug('Reloading config file as files have been modified.')
        self.config.plugin, self.config.general = self.config.reload()
        logging.debug('Reloading plugins as files have been modified.')
        self.loader = plugin_manager.PluginLoader(
            self._plugin_path, self.config.plugin)
        self._plugin_mod_time = os.path.getmtime(self._plugin_path)
        self._config_mod_time = os.path.getmtime(self._config_file_path)

    def run_plugins(self):
        """
        Creates a thread for each plugin and lets the thread_manager handle it.
        """
        for obj in self.loader.objects:
            # Reserve a slot in the output_dict in order to ensure that the
            # items are in the correct order.
            self.output_dict[obj.output_options['name']] = None
            self.thread_manager.add_thread(obj.main, obj.options['interval'])

    def run(self):
        """
        Monitors if the config file or plugins are updated. Also outputs the
        JSON data generated by the plugins, without needing to poll the threads.
        """
        self.run_plugins()
        while True:
            # Reload plugins and config if either the config file or plugin
            # directory are modified.
            if self._config_mod_time != os.path.getmtime(self._config_file_path) or \
                    self._plugin_mod_time != os.path.getmtime(self._plugin_path):
                self.thread_manager.kill_all_threads()
                self.output_dict.clear()
                self.reload()
                self.run_plugins()
            self.output_to_bar(json.dumps(self._remove_empty_output()))
            time.sleep(self.config.general['interval'])

    def _remove_empty_output(self):
        """
        If plugins haven't been initialised and therefore not sending output or
        their output is None, there is no reason to take up extra room on the
        bar.
        """
        output = []
        for key in self.output_dict:
            if self.output_dict[key] is not None and 'full_text' in self.output_dict[key]:
                output.append(self.output_dict[key])
        return output

    def handle_events(self):
        """
        An event handler that processes events from stdin and calls the on_click
        function of the respective object. This function is run in another
        thread, so as to not stall the main thread.
        """
        for event in sys.stdin:
            if event.startswith('['):
                continue
            name = json.loads(event.lstrip(','))['name']
            for obj in self.loader.objects:
                if obj.output_options['name'] == name:
                    obj.on_click(json.loads(event.lstrip(',')))

    class LoggingWriter():
        """
        A simple class that provides a file like interface to the logging
        utility. Allows stderr to be redirected to logging.
        """
        def __init__(self, filename, logging_level):
            # FIXME: This formatting string overwrites the one defined
            # before, has been mentioned on the internet as being a bug
            # in th elogging module, will investigate later.
            formatting = ('[%(asctime)s] - %(levelname)s - %(filename)s -'
                            ' %(threadName)s - %(funcName)s - %(message)s',
                            '%d/%m/%Y %I:%M:%S %p')
            self.logger = setup_file_logger(filename, formatting, logging_level)
            self.logging_level = logging_level

        def write(self, message):
            """
            Called once stderr is replaced and data needs to be written.
            """
            if message != '\n':
                self.logger.log(self.logging_level, message)



import subprocess
import json


class Plugin():
    """
    A base class that should be inherited by all other plugins.
    It handles defaults, replacing options and managing lower level
    settings that a plugin developer need not bother with.

    config: A dictionary obtained from the config parser that has the
    user's configuration, as defined in their config file.
    """
    def __init__(self, config):
        self.output_options = {
            'color': None,
            'min_width': None,
            'align': None,
            'name': None,
            'urgent': None,
            'seperator': None,
            'seperator_block_width': None}
        # Replace default values with user defined ones.
        self.options.update(config)
        # Allow output options to be overwritten by the user.
        self.output_options.update((k, v) for k, v in config.items() if k in
                                   self.output_options)

    def output(self, full_text, short_text):
        """
        Output all of the options and data for a segment.

        full_text: A string representing the data that should be output to i3bar.
        short_text: A more concise version of full_text, in case there is minimal
        room on the i3bar.
        """
        self.output_options.update({'full_text': full_text, 'short_text': short_text})
        self.output_options = {k: v for k, v in self.output_options.items() if v}
        return self.output_options

    def on_click(self, event):
        """
        A function that should be overwritten by a plugin that wishes to react
        to events, if it wants to perform any action other than running the
        supplied command related to a button.

        event: A dictionary passed from i3bar (after being decoded from JSON)
        that has the folowing format:

        event = {'name': 'my_plugin', 'x': 231, 'y': 423}
        Note: It is also possible to have an instance key, but i3situation
        doesn't set it.
        """
        if event['button'] == 1 and 'button1' in self.options:
            subprocess.call(self.options['button1'].split())
        elif event['button'] == 2 and 'button2' in self.options:
            subprocess.call(self.options['button2'].split())
        elif event['button'] == 3 and 'button3' in self.options:
            subprocess.call(self.options['button3'].split())



#!/usr/bin/python3
import os
from setuptools import setup
from setuptools import find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(name='i3situation',
      packages=find_packages(),
      version='1.0.3',
      description='A replacement for i3status that allows for the use of plugins.',
      author='Harvey Hunt',
      url='https://github.com/HarveyHunt/i3situation',
      author_email='harveyhuntnexus@gmail.com',
      license="GPLv3",
      keywords="python3 i3situation i3 i3wm i3status i3bar json",
      install_requires=['requests'],
      long_description=read('README.md'),
      entry_points={'console_scripts': ['i3situation=i3situation.main:main']})



from i3situation.plugins._plugin import Plugin

__all__ = 'TextPlugin'


class TextPlugin(Plugin):

    def __init__(self, config):
        self.options = {'text': '', 'interval': 1}
        super().__init__(config)

    def main(self):
        return self.output(self.options['text'], self.options['text'])



import os
import errno
import configparser


class IncompleteConfigurationFile(ValueError):
    pass


class Config():
    """
    Provides a simplified interface to the configuration of the application.

    Accepts a list of folder_locations that are then checked for validity.
    """
    def __init__(self, folder_locations):
        for path in folder_locations:
            if os.path.isdir(path):
                self._folder_path = path
                break
        else:
            self._folder_path = folder_locations[0]
            self._touch_dir(self._folder_path)
        self.plugin_path = os.path.join(self._folder_path, 'plugins')
        if not os.path.exists(self.plugin_path):
            self._touch_dir(self.plugin_path)
        self.config_file_path = os.path.join(self._folder_path, 'config')
        if not os.path.exists(self.config_file_path):
            self.create_default_config()
        self.plugin, self.general = self.reload()

    def _touch_dir(self, path):
        """
        A helper function to create a directory if it doesn't exist.

        path: A string containing a full path to the directory to be created.
        """
        try:
            os.makedirs(path)
        except OSError as e:
            if e.errno != errno.EEXIST:
                raise

    def create_default_config(self):
        s = '[general]\ninterval = 1\nlogging_level = ERROR\n' \
            'log_file = ~/.config/i3situation/log.txt\ncolors = true'
        with open(self.config_file_path, 'w') as f:
            f.write(s)

    def reload(self):
        """
        Reload the configuration from the file. This is in its own function
        so that it can be called at any time by another class.
        """
        self._conf = configparser.ConfigParser()
        # Preserve the case of sections and keys.
        self._conf.optionxform = str
        self._conf.read(self.config_file_path)
        if 'general' not in self._conf.keys():
            raise IncompleteConfigurationFile('Missing the general section')
        general = self._replace_data_types(dict(self._conf.items('general')))
        self._conf.remove_section('general')
        plugin = []
        for section in self._conf.sections():
            plugin.append(dict(self._conf.items(section)))
            plugin[-1].update({'name': section})
            plugin[-1] = self._replace_data_types(plugin[-1])
        return (plugin, general)

    @staticmethod
    def _replace_data_types(dictionary):
        """
        Replaces strings with appropriate data types (int, boolean).
        Also replaces the human readable logging levels with the integer form.

        dictionary: A dictionary returned from the config file.
        """
        logging_levels = {'NONE': 0, 'NULL': 0, 'DEBUG': 10, 'INFO': 20,
                'WARNING': 30, 'ERROR': 40, 'CRITICAL': 50}
        for k, v in dictionary.items():
            if v in ['true', 'True', 'on']:
                dictionary[k] = True
            elif v in ['false', 'False', 'off']:
                dictionary[k] = False
            elif k == 'log_file' and '~' in v:
                dictionary[k] = v.replace('~', os.path.expanduser('~'))
            elif v in logging_levels:
                dictionary[k] = logging_levels[v]
            elif isinstance(v, str) and v.isnumeric():
                dictionary[k] = int(v)
            elif ',' in v:
                dictionary[k] = [x.strip() for x in v.split(',')]
        return dictionary
